<?php

namespace Drupal\gradient\Element;

use Drupal\Core\Render\Attribute\FormElement;
use Drupal\Core\Render\Element\FormElementBase;

/**
 * Provides a form element for creating gradients.
 *
 * Properties:
 * - #default_value: A string.
 *
 * Usage example:
 *
 * @code
 * $form['gradient'] = [
 *   '#type' => 'gradient',
 *   '#title' => $this->t('Gradient Builder'),
 * ];
 * @endcode
 *
 */
#[FormElement('gradient')]
class Gradient extends FormElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#default_value' => NULL,
      '#pre_render' => [
        [$class, 'preRenderGradient'],
      ],
      '#process' => [
        [$class, 'processGradient'],
      ],
      '#element_validate' => [
        [$class, 'validateGradient'],
      ],
      '#theme' => 'gradient',
      '#theme_wrappers' => ['form_element'],
    ];
  }

  public static function preRenderGradient($element) {
    $element['#builder_id'] = \Drupal::service('uuid')->generate();
    return $element;
  }

  /**
   * Process gradient.
   */
  public static function processGradient(&$element, &$form_state, &$complete_form) {
    $element['#attached']['library'][] = 'gradient/builder';
    if (!isset($element['#value'])) {
      $element['#value'] = $element['#default_value'];
    }
    $element['#attributes'] = [
      'type' => 'hidden',
      'name' => 'gradient',
    ];
    if (isset($element['#value'])) {
      $element['#attributes']['value'] = $element['#value'];
    }

    return $element;
  }

  /**
   * Validate gradient.
   */
  public static function validateGradient(&$element, &$form_state, &$complete_form) {
    $value = $element['#value'];
    if (empty($value)) {
      // @todo validate #[0-9a-fA-F]{6}
      $form_state->setError($element, t('The gradient cannot be empty.'));
    }
  }

}
